
set(COMPILER_TEST_DIR ${CMAKE_BINARY_DIR}/CMakeFiles/compilerTest)
file(WRITE ${COMPILER_TEST_DIR}/CMakeLists.txt 
    "project(compilerTest)
     cmake_minimum_required(VERSION 2.8)
     add_definitions(-std=c++11)
# VARIADIC CLASS_TEMPLATES
     add_executable(test_variadic_templ_class test_variadic_templ_class.cpp)
     target_link_libraries(test_variadic_templ_class)
# VARIADIC FUNCTION TEMPLATES
     add_executable(test_variadic_templ_fct test_variadic_templ_fct.cpp)
     target_link_libraries(test_variadic_templ_fct)
# ALIAS TEMPLATES
     add_executable(test_alias_templates test_alias_templates.cpp)
     target_link_libraries(test_alias_templates)
# DECLTYPE
     add_executable(test_decltype test_decltype.cpp)
     target_link_libraries(test_decltype)
# AUTO SPECIFIER
     add_executable(test_auto test_auto.cpp)
     target_link_libraries(test_auto)
# CONSTEXPR
     add_executable(test_constexpr test_constexpr.cpp)
     target_link_libraries(test_constexpr)
# DELEGATING CONSTRUCTORS
     add_executable(test_delegating_constructors test_delegating_constructors.cpp)
     target_link_libraries(test_delegating_constructors)
# RANGE-BASED FOR LOOPS
     add_executable(test_range_based_for test_range_based_for.cpp)
     target_link_libraries(test_range_based_for)")
     
file(WRITE ${COMPILER_TEST_DIR}/test_variadic_templ_class.cpp 
     "template<class... Ts> struct A{}; int main(){A<int, double> a;}")
file(WRITE ${COMPILER_TEST_DIR}/test_variadic_templ_fct.cpp 
     "template<class... Ts> void foo(Ts... ts){}; int main(){foo(1, 2.0, 3.0f);}")
file(WRITE ${COMPILER_TEST_DIR}/test_alias_templates.cpp 
     "template<class T> struct A{}; template<class T> using B=A<T>; int main(){B<int> b;}")
file(WRITE ${COMPILER_TEST_DIR}/test_decltype.cpp 
     "int main(){decltype(1) a = 1;}")
file(WRITE ${COMPILER_TEST_DIR}/test_auto.cpp 
     "int main(){auto a = 1;}")
file(WRITE ${COMPILER_TEST_DIR}/test_constexpr.cpp 
     "constexpr int foo(){return 1;}; int main(){static constexpr int f = foo();}")
file(WRITE ${COMPILER_TEST_DIR}/test_delegating_constructors.cpp 
     "struct A{ A(){} A(int) : A(){} }; int main(){ A a(1);}")
file(WRITE ${COMPILER_TEST_DIR}/test_range_based_for.cpp 
     "int main(){ int vec[10]; for (int& elem : vec) { elem = 1; } }")
     
     
set(COMPILER_CXX11_FEATURES "")

try_compile(TEST_VARIADIC_TEMPL_CLASS ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_variadic_templ_class)
try_compile(TEST_VARIADIC_TEMPL_FCT ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_variadic_templ_fct)
if (TEST_VARIADIC_TEMPL_CLASS AND TEST_VARIADIC_TEMPL_FCT)
  message("Compiler supports variadic class/ function templates")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_VARIADIC_TEMPLATES=1")
endif()

try_compile(TEST_ALIAS_TEMPLATES ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_alias_templates)
if (TEST_ALIAS_TEMPLATES)
  message("Compiler supports alias-templates")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_ALIAS_TEMPLATES=1")
endif()

try_compile(TEST_DECLTYPE ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_decltype)
if (TEST_DECLTYPE)
  message("Compiler supports decltype")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_DECLTYPE=1")
endif()

try_compile(TEST_AUTO_SPECIFIER ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_auto)
if (TEST_AUTO_SPECIFIER)
  message("Compiler supports auto")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_AUTO=1")
endif()

try_compile(TEST_CONSTEXPR ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_constexpr)
if (TEST_CONSTEXPR)
  message("Compiler supports constexpr")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_CONSTEXPR=1")
endif()

try_compile(TEST_DELEGATING_CONSTRUCTORS ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_delegating_constructors)
if (TEST_DELEGATING_CONSTRUCTORS)
  message("Compiler supports delegating constructors")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_DELEGATING_CONSTRUCTORS=1")
endif()

try_compile(TEST_RANGED_BASED_FOR ${COMPILER_TEST_DIR} ${COMPILER_TEST_DIR} compilerTest test_range_based_for)
if (TEST_RANGED_BASED_FOR)
  message("Compiler supports range-based for loops")
  list(APPEND COMPILER_CXX11_FEATURES "-DHAS_RANGE_BASED_FOR=1")
endif()