/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/



/** \file GlobalElementNumbering.h */

#ifndef AMDIS_GLOBALELEMENTNUMBERING_H
#define AMDIS_GLOBALELEMENTNUMBERING_H

#include <map>
#include "AMDiS_fwd.h"

namespace AMDiS {

  /** \ingroup Parallel
   * \brief 
   * Creates a global element numbering.
   */
  class GlobalElementNumbering
  {
  public:
    /// Constructor
    GlobalElementNumbering(MeshStructure *compositeStructure,
			   Mesh *localMesh);

    /// Returns the local index corresponding to local index.
    int getLocalIndex(int globalIndex);

    /// Returns the global index corresponing to local index.
    int getGlobalIndex(int localIndex);

  protected:
    /// Maps each local index to its global index.
    std::map<int, int> localToGlobal_;

    /// Stores local index for each global index.
    std::map<int, int> globalToLocal_;
  };

}

#endif
