/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


namespace AMDiS {

  template <typename T>
  void Operator::addSecondOrderTerm(T *term)
  {
    secondOrder.push_back(term);
    term->operat = this;
    for (std::set<const FiniteElemSpace*>::iterator it = term->getAuxFeSpaces().begin();
	 it != term->getAuxFeSpaces().end(); ++it)
      auxFeSpaces.insert(*it);
  }

  template <typename T>
  void Operator::addFirstOrderTerm(T *term,
				   FirstOrderType type)
  {
    if (type == GRD_PSI) {
      firstOrderGrdPsi.push_back(term);
    } else {
      firstOrderGrdPhi.push_back(term);
    }
    term->operat = this;
    for (std::set<const FiniteElemSpace*>::iterator it = term->getAuxFeSpaces().begin();
	 it != term->getAuxFeSpaces().end(); ++it)
      auxFeSpaces.insert(*it);
  }

  template <typename T>
  void Operator::addZeroOrderTerm(T *term)
  {
    zeroOrder.push_back(term);
    term->operat = this;
    for (std::set<const FiniteElemSpace*>::iterator it = term->getAuxFeSpaces().begin();
	 it != term->getAuxFeSpaces().end(); ++it)
      auxFeSpaces.insert(*it);
  }

}
