/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/



/** \file VertexVector.h */

#ifndef AMDIS_VERTEXVECTOR_H
#define AMDIS_VERTEXVECTOR_H

#include "DOFVector.h"

namespace AMDiS {

  class VertexVector : public DOFVectorDOF
  {
  public:
    class Iterator : public DOFIterator<DegreeOfFreedom> {
    public:
      Iterator(VertexVector *c, DOFIteratorType type)
	: DOFIterator<DegreeOfFreedom>(const_cast<DOFAdmin*>(c->getAdmin()), 
				       dynamic_cast<DOFIndexed<DegreeOfFreedom>*>(c), 
				       type)
      {}   
    };

    VertexVector(const DOFAdmin *admin, std::string name);

    ~VertexVector();

    void freeDofIndex(DegreeOfFreedom dof)
    {
      FUNCNAME_DBG("VertexVector::freeDofIndex()");
      
      TEST_EXIT_DBG(dof < static_cast<int>(vec.size()))("Should not happen!\n");

      vec[dof] = dof;
    }

    const DOFAdmin *getAdmin() 
    { 
      return admin; 
    }

    void resize(int size) 
    {
      int oldSize = static_cast<int>(vec.size());
      vec.resize(size);
      for (int i = oldSize; i < size; i++)
	vec[i] = i;
    }

    void set(DegreeOfFreedom val);

    void compressDofContainer(int size, std::vector<DegreeOfFreedom> &newDOF) 
    {
      DOFContainer::compressDofContainer(size, newDOF);
      int totalSize = getAdmin()->getSize();
      for (int i = size; i < totalSize; i++)
	vec[i] = i;
    }

    void changeDofIndices(std::map<DegreeOfFreedom, DegreeOfFreedom>& dofIndexMap);

    void print();

  protected:
    const DOFAdmin *admin;
  };

}

#endif
